#' PP diagnostic for fitted truncated gamma model

#' @param r Values of R|W that exceed the threshold r0w
#' @param w Values of W that correspond to the threshold exceedances of R
#' @param r0w The threshold function r_0(w) for each value of W supplied
#' @param par The vector of fitted parameters (in the same order as obtained from fit.geometricMVE.2d)
#' @param gfun gauge function to be used in the likelihood. Can use inbuilt examples or specify own function of the form gauge<-function(xy,par){}. Leave blank if want to mix gauges additively.
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in par
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in par
#'
#' @return PP plot with 95\% tolerance intervals
#'
#' @author Jenny Wadsworth
#' @export

ppdiag.2d<-function(r,w,r0w,par,gfun=c(gauge_rvad, gauge_gaussian,gauge_invlogistic, gauge_square),
                  add.gauge=FALSE,gauge1=c(gauge_rvad, gauge_gaussian,gauge_invlogistic, gauge_square),
                  gauge2=c(gauge_rvad, gauge_gaussian,gauge_invlogistic, gauge_square),par1ind,par2ind)
{

  if(!add.gauge){
  num<-pgamma(r,shape=par[1],
              rate=apply(cbind(w,1-w),1,gfun,par=par[2:length(par)]),lower.tail = F)
  den<-pgamma(r0w,shape=par[1],
              rate=apply(cbind(w,1-w),1,gfun,par=par[2:length(par)]),lower.tail=F)
  } else{
    ms<-additivegauge.scaling(gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind])
    num<-pgamma(r,shape=par[1],
                rate=apply(cbind(w,1-w),1,additivegauge.rescale,par1=par[par1ind],par2=par[par2ind],ms=ms,gauge1=gauge1,gauge2=gauge2,weight=par[length(par)]),lower.tail = F)
    den<-pgamma(r0w,shape=par[1],
                rate=apply(cbind(w,1-w),1,additivegauge.rescale,par1=par[par1ind],par2=par[par2ind],ms=ms,gauge1=gauge1,gauge2=gauge2,weight=par[length(par)]),lower.tail = F)
  }
  nr<-length(r)
  plot(c(1:nr)/(nr+1),
       sort(1-num/den),xlab="empirical",ylab="model",pch=20,cex=0.8)
  abline(a=0,b=1)

  Ulow<-sapply(1:nr,function(i){qbeta(0.025,i,nr+1-i)})
  Uup<-sapply(1:nr,function(i){qbeta(0.975,i,nr+1-i)})

  lines(c(1:nr)/(nr+1),Ulow,lty=2)
  lines(c(1:nr)/(nr+1),Uup,lty=2)
}


#' QQ diagnostic for fitted truncated gamma model - puts PP plot on any desired quantile scale (default is standard exponential)

#' @param r Values of R|W that exceed the threshold r0w
#' @param w Values of W that correspond to the threshold exceedances of R
#' @param r0w The threshold function r_0(w) for each value of W supplied
#' @param par The vector of fitted parameters (in the same order as obtained from fit.geometricMVE.2d)
#' @param gfun gauge function to be used in the likelihood. Can use inbuilt examples or specify own function of the form gauge<-function(xy,par){}. Leave blank if want to mix gauges additively.
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in par
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in par
#' @param quantilefn a quantile function to specify the desired scale. Defaults to qexp.
#'
#' @return QQ plot with 95\% tolerance intervals
#'
#' @author Jenny Wadsworth
#' @export

qqdiag.2d<-function(r,w,r0w,par,gfun=c(gauge_rvad, gauge_gaussian,gauge_invlogistic, gauge_square),
                     add.gauge=FALSE,gauge1=c(gauge_rvad, gauge_gaussian,gauge_invlogistic, gauge_square),
                     gauge2=c(gauge_rvad, gauge_gaussian,gauge_invlogistic, gauge_square),par1ind,par2ind,quantilefn=qexp)
{

  if(!add.gauge){
    num<-pgamma(r,shape=par[1],
                rate=apply(cbind(w,1-w),1,gfun,par=par[2:length(par)]),lower.tail = F)
    den<-pgamma(r0w,shape=par[1],
                rate=apply(cbind(w,1-w),1,gfun,par=par[2:length(par)]),lower.tail=F)
  } else{
    ms<-additivegauge.scaling(gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind])
    num<-pgamma(r,shape=par[1],
                rate=apply(cbind(w,1-w),1,additivegauge.rescale,par1=par[par1ind],par2=par[par2ind],ms=ms,gauge1=gauge1,gauge2=gauge2,weight=par[length(par)]),lower.tail = F)
    den<-pgamma(r0w,shape=par[1],
                rate=apply(cbind(w,1-w),1,additivegauge.rescale,par1=par[par1ind],par2=par[par2ind],ms=ms,gauge1=gauge1,gauge2=gauge2,weight=par[length(par)]),lower.tail = F)
  }
  nr<-length(r)
  plot(quantilefn(c(1:nr)/(nr+1)),
       quantilefn(sort(1-num/den)),xlab="empirical",ylab="model",pch=20,cex=0.8)
  abline(a=0,b=1)

  Ulow<-sapply(1:nr,function(i){qbeta(0.025,i,nr+1-i)})
  Uup<-sapply(1:nr,function(i){qbeta(0.975,i,nr+1-i)})

  lines(quantilefn(c(1:nr)/(nr+1)),quantilefn(Ulow),lty=2)
  lines(quantilefn(c(1:nr)/(nr+1)),quantilefn(Uup),lty=2)
}


#====================================================================

#' PP diagnostic for fitted truncated gamma model

#' @param r Values of R|W that exceed the threshold r0w
#' @param w Values of W that correspond to the threshold exceedances of R
#' @param r0w The threshold function r_0(w) for each value of W supplied
#' @param par The vector of fitted parameters (in the same order as obtained from fit.geometricMVE.3d)
#' @param gfun gauge function to be used in the likelihood. Can use inbuilt examples or specify own function of the form gauge<-function(xy,par){}. Leave blank if want to mix gauges additively.
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in par
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in par
#'
#' @return PP plot with 95\% tolerance intervals
#'
#' @author Jenny Wadsworth
#' @export

ppdiag.3d<-function(r,w,r0w,par,gfun,
                    add.gauge=FALSE,gauge1,gauge2,par1ind,par2ind)
{
  if(dim(w)[2]==2){w<-cbind(w,1-w[,1]-w[,2])}
  if(!add.gauge){
    num<-pgamma(r,shape=par[1],
                rate=apply(w,1,gfun,par=par[2:length(par)]),lower.tail = F)
    den<-pgamma(r0w,shape=par[1],
                rate=apply(w,1,gfun,par=par[2:length(par)]),lower.tail=F)
  } else{
    wgrid<-create.wgrid.3d(70)
    ms<-additivegauge.scaling.3d(gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind],wgrid=wgrid)
    num<-pgamma(r,shape=par[1],
                rate=apply(w,1,additivegauge.rescale.3d,
                           par1=par[par1ind],par2=par[par2ind],ms=ms,
                           gauge1=gauge1,gauge2=gauge2,weight=par[length(par)]),lower.tail = F)
    den<-pgamma(r0w,shape=par[1],
                rate=apply(w,1,additivegauge.rescale.3d,
                           par1=par[par1ind],par2=par[par2ind],ms=ms,
                           gauge1=gauge1,gauge2=gauge2,weight=par[length(par)]),lower.tail = F)
  }
  nr<-length(r)
  plot(c(1:nr)/(nr+1),
       sort(1-num/den),xlab="empirical",ylab="model",pch=20,cex=0.8)
  abline(a=0,b=1)

  Ulow<-sapply(1:nr,function(i){qbeta(0.025,i,nr+1-i)})
  Uup<-sapply(1:nr,function(i){qbeta(0.975,i,nr+1-i)})

  lines(c(1:nr)/(nr+1),Ulow,lty=2)
  lines(c(1:nr)/(nr+1),Uup,lty=2)
}


#' QQ diagnostic for fitted truncated gamma model - puts PP plot on any desired quantile scale (default is standard exponential)

#' @param r Values of R|W that exceed the threshold r0w
#' @param w Values of W that correspond to the threshold exceedances of R
#' @param r0w The threshold function r_0(w) for each value of W supplied
#' @param par The vector of fitted parameters (in the same order as obtained from fit.geometricMVE.3d)
#' @param gfun gauge function to be used in the likelihood. Can use inbuilt examples or specify own function of the form gauge<-function(xy,par){}. Leave blank if want to mix gauges additively.
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in par
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in par
#' @param quantilefn a quantile function to specify the desired scale. Defaults to qexp.
#'
#' @return QQ plot with 95\% tolerance intervals
#'
#' @author Jenny Wadsworth
#' @export

qqdiag.3d<-function(r,w,r0w,par,gfun,
                    add.gauge=FALSE,gauge1,gauge2,par1ind,par2ind,quantilefn=qexp)
{

  if(dim(w)[2]==2){w<-cbind(w,1-w[,1]-w[,2])}
  if(!add.gauge){
    num<-pgamma(r,shape=par[1],
                rate=apply(w,1,gfun,par=par[2:length(par)]),lower.tail = F)
    den<-pgamma(r0w,shape=par[1],
                rate=apply(w,1,gfun,par=par[2:length(par)]),lower.tail=F)
  } else{
    wgrid<-create.wgrid.3d(70)
    ms<-additivegauge.scaling.3d(gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind],wgrid=wgrid)
    num<-pgamma(r,shape=par[1],
                rate=apply(w,1,additivegauge.rescale.3d,
                           par1=par[par1ind],par2=par[par2ind],ms=ms,
                           gauge1=gauge1,gauge2=gauge2,weight=par[length(par)]),lower.tail = F)
    den<-pgamma(r0w,shape=par[1],
                rate=apply(w,1,additivegauge.rescale.3d,
                           par1=par[par1ind],par2=par[par2ind],ms=ms,
                           gauge1=gauge1,gauge2=gauge2,weight=par[length(par)]),lower.tail = F)
  }
  nr<-length(r)
  plot(quantilefn(c(1:nr)/(nr+1)),
       quantilefn(sort(1-num/den)),xlab="empirical",ylab="model",pch=20,cex=0.8)
  abline(a=0,b=1)

  Ulow<-sapply(1:nr,function(i){qbeta(0.025,i,nr+1-i)})
  Uup<-sapply(1:nr,function(i){qbeta(0.975,i,nr+1-i)})

  lines(quantilefn(c(1:nr)/(nr+1)),quantilefn(Ulow),lty=2)
  lines(quantilefn(c(1:nr)/(nr+1)),quantilefn(Uup),lty=2)
}


#==============================================================

#' Plot unit level set of fitted parametric gauge function

#' @param wpts sequence of w values at which to plot gauge function
#' @param gfun gauge function
#' @param par parameters of the gauge function specified by gfun
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in par
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in par
#' @param xlab Graphical parameter - label of x axis
#' @param ylab Graphical parameter - label of y axis
#' @param col Graphical parameter - colour of line
#' @param type Graphical parameter - type of line
#' @param lwd Graphical parameter - width of line
#' @param polygon Logical - draw a shaded version of the limit set?
#' @param polygoncol - Graphical parameter - colour of the limit set shading
#' @author Jenny Wadsworth
#' @export



plotfittedgauge.2d<-function(wpts,gfun,par,add=FALSE,add.gauge=FALSE,
                             gauge1,gauge2,par1ind,par2ind,xlab="x",ylab="y",
                             col=2,type="l",lwd=1,polygon=FALSE,polygoncol=rgb(0.5,0.5,0.5,alpha=0.5))
{
  if(add.gauge)
  {
    ms<-additivegauge.scaling(gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind])
    gfun<-function(xy,par){
      additivegauge.rescale(xy = xy,gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind],ms=ms)
    }
  }

  if(add){
    lines(wpts/apply(cbind(wpts,1-wpts),1,gfun,par=par),(1-wpts)/apply(cbind(wpts,1-wpts),1,gfun,par=par),col=col, typ=type, lwd=lwd)
    if(polygon){
      polygon(c(0,wpts/apply(cbind(wpts,1-wpts),1,gfun,par=par),0),c(0,(1-wpts)/apply(cbind(wpts,1-wpts),1,gfun,par=par),0),col=polygoncol,border = NULL)
    }
  }
  else{
    plot(wpts/apply(cbind(wpts,1-wpts),1,gfun,par=par),(1-wpts)/apply(cbind(wpts,1-wpts),1,gfun,par=par), typ=type, col=col, xlab=xlab, ylab=ylab, lwd=lwd)
    if(polygon)
    {
      polygon(c(0,wpts/apply(cbind(wpts,1-wpts),1,gfun,par=par),0),c(0,(1-wpts)/apply(cbind(wpts,1-wpts),1,gfun,par=par),0),col=polygoncol,border = NULL)
    }
  }
}

# plotfittedgauge.2d<-function(wpts,gfun,par,add=FALSE,add.gauge=FALSE,gauge1,gauge2,par1ind,par2ind,xlab="x",ylab="y",col=2,type="l",lwd=1)
# {
#   if(add.gauge)
#   {
#     ms<-additivegauge.scaling(gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind])
#     gfun<-function(xy,par){
#       additivegauge.rescale(xy = xy,gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind],ms=ms)
#     }
#   }
#
#   if(add){
#     lines(wpts/apply(cbind(wpts,1-wpts),1,gfun,par=par),(1-wpts)/apply(cbind(wpts,1-wpts),1,gfun,par=par),col=col, typ=type, lwd=lwd)
#   }
#   else{
#     plot(wpts/apply(cbind(wpts,1-wpts),1,gfun,par=par),(1-wpts)/apply(cbind(wpts,1-wpts),1,gfun,par=par), typ=type, col=col, xlab=xlab, ylab=ylab, lwd=lwd)
#   }
# }


# plotfittedgauge.2d<-function(wpts,gfun,par,add=FALSE)
# {
#   if(add){
#     lines(wpts/apply(cbind(wpts,1-wpts),1,gfun,par=par),(1-wpts)/apply(cbind(wpts,1-wpts),1,gfun,par=par),col=2)
#   }
#   else{
#     plot(wpts/apply(cbind(wpts,1-wpts),1,gfun,par=par),(1-wpts)/apply(cbind(wpts,1-wpts),1,gfun,par=par), typ="l", col=2, xlab="x", ylab="y")
#   }
# }



#==============================================================

#' Plot unit level set of fitted parametric gauge function

#' @param grid length of grid used to plot gauge function - higher values give better resolution plots, but take longer
#' @param gfun gauge function
#' @param par parameters of the gauge function specified by gfun
#' @param add logical - should the plot be added to an existing plot?
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in par
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in par
#' @param xlab Graphical parameter - label of x axis
#' @param ylab Graphical parameter - label of y axis
#' @param zlab Graphical parameter - label of z axis
#' @param col Graphical parameter - colour of surface
#' @param alphaval - Graphical parameter - transparency of surface
#'
#' @author Jenny Wadsworth
#' @export


plotfittedgauge.3d<-function(grid=50,gfun,par,add=FALSE,add.gauge=FALSE,gauge1,gauge2,
                             par1ind,par2ind, xlab="x",ylab="y",zlab="z",col=4,alphaval=0.5)
{
  require(misc3d)
  if(add.gauge)
  {
    wgrid<-create.wgrid.3d(70)
    ms<-additivegauge.scaling.3d(gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind],wgrid=wgrid)
    gfun.plot<-function(x,y,z)
    {
      additivegauge.rescale.3d(xyz=c(x,y,z),gauge1=gauge1,gauge2=gauge2,weight=par[length(par)],par1=par[par1ind],par2=par[par2ind],ms=ms)
    }
  } else{
    gfun.plot<-function(x,y,z)
    {
    gfun(xyz=c(x,y,z),par=par)
    }
  }
  gfun.plot<-Vectorize(gfun.plot)
  x1<-seq(0,1,len=grid)
  if(add){
    misc3d::contour3d(f=gfun.plot,level=c(1),x=x1,y=x1,z=x1,color=col,alpha=alphaval,add=T)
  }
  else{
    require(rgl)
    misc3d::contour3d(f=gfun.plot,level=c(1),x=x1,y=x1,z=x1,color=col,alpha=alphaval)
    rgl::axes3d()
    rgl::title3d(xlab=xlab,ylab=ylab,zlab=zlab)
  }
}


# plotfittedgauge.3d<-function(grid=50,gfun,par,add=FALSE)
# {
#   require(misc3d)
#   gfun.plot<-function(x,y,z)
#   {
#     gfun(xyz=c(x,y,z),par=par)
#   }
#   gfun.plot<-Vectorize(gfun.plot)
#   x1<-seq(0,1,len=grid)
#   if(add){
#     misc3d::contour3d(f=gfun.plot,level=c(1),x=x1,y=x1,z=x1,color=4,alpha=0.5,add=T)
#   }
#   else{
#     require(rgl)
#     misc3d::contour3d(f=gfun.plot,level=c(1),x=x1,y=x1,z=x1,color=4,alpha=0.5)
#     rgl::axes3d()
#     rgl::title3d(xlab="x",ylab="y",zlab="z")
#   }
# }

