#' Negative log likelihood function for truncated gamma model for R|W (bivariate data)

#' @param psi vector of parameters. First component is the gamma shape parameter
#' @param r Values of R|W=w that exceed the threshold r0w
#' @param w Values of W that correspond to the threshold exceedances of R
#' @param r0w The threshold function r_0(w) for each value of W supplied
#' @param gfun gauge function to be used in the likelihood. Can use inbuilt examples or specify own function of the form gauge<-function(xy,par){}. Leave blank if want to mix gauges additively.
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in psi
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in psi
#' @param pos.par logical. Set to TRUE if all parameters are positive (avoids some warnings in optimization)
#' @param lower.limit optional vector of lower limits on parameters (should be same length as psi)
#' @param upper.limit optional vector of upper limits on parameters (should be same length as psi)
#'
#' @return negative log-likelihood value
#'
#' @author Jenny Wadsworth


rw.gamma.lik.2d<-function(psi,r,w,r0w,gfun,add.gauge=FALSE,gauge1,gauge2,par1ind,par2ind,pos.par,lower.limit,upper.limit)
{
  if(pos.par){if(any(psi<=0)){return(10e10)}}
  if(!is.null(lower.limit)){if(any(psi<lower.limit)){return(10e10)}}
  if(!is.null(upper.limit)){if(any(psi>upper.limit)){return(10e10)}}
  if(add.gauge)
  {
    ms<-additivegauge.scaling(gauge1=gauge1,gauge2=gauge2,weight=psi[length(psi)],par1=psi[par1ind],par2=psi[par2ind])
    rate<-apply(cbind(w,1-w),1,additivegauge.rescale,par1=psi[par1ind],par2=psi[par2ind],ms=ms,gauge1=gauge1,gauge2=gauge2,weight=psi[length(psi)])
    ll1<-dgamma(r,shape=psi[1],rate=rate,log=T)
    ll2<-log(pgamma(r0w,shape=psi[1],rate=rate,lower.tail=F))
  }
  else{
    rate<-apply(cbind(w,1-w),1,gfun,par=psi[2:length(psi)])
    ll1<-dgamma(r,shape=psi[1],rate=rate,log=T)
    ll2<-log(pgamma(r0w,shape=psi[1],rate=rate,lower.tail=F))
  }
  return(-sum(ll1)+sum(ll2))
}

#============================================================================
#' Negative log likelihood function for truncated gamma model for R|W (trivariate data)

#' @param psi vector of parameters. First component is the gamma shape parameter
#' @param r Values of R|W=w that exceed the threshold r0w
#' @param w Values of W that correspond to the threshold exceedances of R (2 or 3 column matrix)
#' @param r0w The threshold function r_0(w) for each value of W supplied
#' @param gfun gauge function to be used in the likelihood. Can use inbuilt examples or specify own function of the form gauge<-function(xy,par){}. Leave blank if want to mix gauges additively.
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in psi
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in psi
#' @param pos.par logical. Set to TRUE if all parameters are positive (avoids some warnings in optimization).
#' @param lower.limit optional vector of lower limits on parameters (should be same length as psi)
#' @param upper.limit optional vector of upper limits on parameters (should be same length as psi)
#' @param wgrid 3 column matrix containing points that cover the 2-d simplex. Used only if add.gauge=T, to perform numerical rescaling

#' @return negative log-likelihood value
#'
#' @author Jenny Wadsworth


rw.gamma.lik.3d<-function(psi,r,w,r0w,gfun,add.gauge=FALSE,gauge1,gauge2,par1ind,par2ind,pos.par,lower.limit,upper.limit,wgrid)
{
  if(pos.par){if(any(psi<=0)){return(10e10)}}
  if(!is.null(lower.limit)){if(any(psi<lower.limit)){return(10e10)}}
  if(!is.null(upper.limit)){if(any(psi>upper.limit)){return(10e10)}}
  if(dim(w)[2]==2){w<-cbind(w,1-w[,1]-w[,2])}
  if(add.gauge)
  {
    ms<-additivegauge.scaling.3d(gauge1=gauge1,gauge2=gauge2,weight=psi[length(psi)],par1=psi[par1ind],par2=psi[par2ind],wgrid=wgrid)
    rate<-apply(w,1,additivegauge.rescale.3d,par1=psi[par1ind],par2=psi[par2ind],ms=ms,gauge1=gauge1,gauge2=gauge2,weight=psi[length(psi)])
    ll1<-dgamma(r,shape=psi[1],rate=rate,log=T)
    ll2<-log(pgamma(r0w,shape=psi[1],rate=rate,lower.tail=F))
  }
  else{
    rate<-apply(w,1,gfun,par=psi[2:length(psi)])
    ll1<-dgamma(r,shape=psi[1],rate=rate,log=T)
    ll2<-log(pgamma(r0w,shape=psi[1],rate=rate,lower.tail=F))
  }
  return(-sum(ll1)+sum(ll2))
}

#============================================================================

#' Fit truncated gamma model for R|W for bivariate data

#' @param r Values of R|W=w that exceed the threshold r0w
#' @param w Values of W that correspond to the threshold exceedances of R
#' @param r0w The threshold function r_0(w) for each value of W supplied
#' @param gfun gauge function to be used in the likelihood. Can use inbuilt examples or specify own function of the form gauge<-function(xy,par){}. Leave blank if want to mix gauges additively.
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in psi
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in psi
#' @param init.val vector of initial parameter values for optimization
#' @param pos.par logical. Set to TRUE if all parameters are positive (avoids some warnings in optimization).
#' @param lower.limit optional vector of lower limits on parameters (should be same length as init.val)
#' @param upper.limit optional vector of upper limits on parameters (should be same length as init.val)
#' @param ... Additional arguments to pass to optim
#'
#' @return list containing elements "mle", "nllh", "convergence", and additionally "cov" if argument hessian=T is supplied to optim
#'
#' @author Jenny Wadsworth
#' @export



fit.geometricMVE.2d<-function(r,w,r0w,gfun=c(gauge_rvad, gauge_gaussian,gauge_invlogistic, gauge_square),
                              add.gauge=FALSE,gauge1=c(gauge_rvad, gauge_gaussian,gauge_invlogistic, gauge_square),
                              gauge2=c(gauge_rvad, gauge_gaussian,gauge_invlogistic, gauge_square), par1ind=NULL,par2ind=NULL, init.val ,pos.par=TRUE, lower.limit=NULL,upper.limit=NULL,...)
{
  if(add.gauge && (is.null(par1ind)||is.null(par2ind))){
  stop("Please specify positions of parameters for gauge1 and gauge2")
  }
  opt<-optim(rw.gamma.lik.2d, par=init.val,r=r,w=w,r0w=r0w,gfun=gfun,
             add.gauge=add.gauge,gauge1=gauge1,gauge2=gauge2,
             par1ind=par1ind,par2ind=par2ind,pos.par=pos.par,
             lower.limit=lower.limit,upper.limit=upper.limit,...)
  if(is.null(opt$hessian))
  {
    return(list(mle=opt$par,nllh=opt$value,convergence=opt$conv))
  } else{
    return(list(mle=opt$par,cov=solve(opt$hessian),nllh=opt$value,convergence=opt$conv))
  }
}

#============================================================================

#' Fit truncated gamma model for R|W for trivariate data

#' @param r Values of R|W=w that exceed the threshold r0w
#' @param w Values of W that correspond to the threshold exceedances of R (2 or 3 column matrix)
#' @param r0w The threshold function r_0(w) for each value of W supplied
#' @param gfun gauge function to be used in the likelihood. Can use inbuilt examples or specify own function of the form gauge<-function(xy,par){}. Leave blank if want to mix gauges additively.
#' @param add.gauge logical. If TRUE, then we additively mix (with numerical rescaling) the gauge functions specified by gauge1 and gauge2
#' @param gauge1 first gauge function for additive mixing. Use if add.gauge=TRUE
#' @param gauge2 second gauge function for additive mixing. Use if add.gauge=TRUE
#' @param par1ind vector of indices specifying the position of the parameters for gauge1 in psi
#' @param par2ind vector of indices specifying the position of the parameters for gauge2 in psi
#' @param init.val vector of initial parameter values for optimization
#' @param pos.par logical. Set to TRUE if all parameters are positive (avoids some warnings in optimization).
#' @param lower.limit optional vector of lower limits on parameters (should be same length as init.val)
#' @param upper.limit optional vector of upper limits on parameters (should be same length as init.val)
#' @param ... Additional arguments to pass to optim
#'
#' @return list containing elements "mle", "nllh", "convergence", and additionally "cov" if argument hessian=T is supplied to optim
#'
#' @author Jenny Wadsworth
#' @export



fit.geometricMVE.3d<-function(r,w,r0w,gfun,
                              add.gauge=FALSE,gauge1,gauge2,
                              par1ind=NULL,par2ind=NULL,init.val=NULL,pos.par=TRUE,lower.limit=NULL,upper.limit=NULL,...)
{
  if(add.gauge && (is.null(par1ind)||is.null(par2ind))){
    stop("Please specify positions of parameters for gauge1 and gauge2")
  }
  if(add.gauge){wgrid<-create.wgrid.3d(70)}
  opt<-optim(rw.gamma.lik.3d, par=init.val,r=r,w=w,r0w=r0w,gfun=gfun,
             add.gauge=add.gauge,gauge1=gauge1,gauge2=gauge2,pos.par=pos.par,
             par1ind=par1ind,par2ind=par2ind,lower.limit=lower.limit,upper.limit=upper.limit,
             wgrid=wgrid,...)
  if(is.null(opt$hessian))
  {
    return(list(mle=opt$par,nllh=opt$value,convergence=opt$conv))
  } else{
    return(list(mle=opt$par,cov=solve(opt$hessian),nllh=opt$value,convergence=opt$conv))
  }
}
